@@ -1,7 +1,7 @@
 #!/bin/sh
 
 # tst_oathtool.sh - test that oathtool works
-# Copyright (C) 2009-2013 Simon Josefsson
+# Copyright (C) 2009-2014 Simon Josefsson
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -54,6 +54,10 @@ dotest()
     fi
 }
 
+sha1key=3132333435363738393031323334353637383930
+sha256key=3132333435363738393031323334353637383930313233343536373839303132
+sha512key=31323334353637383930313233343536373839303132333435363738393031323334353637383930313233343536373839303132333435363738393031323334
+
 dotest "--version" "fail" fail
 dotest "" "fail" fail
 dotest "-h" "fail" fail
@@ -80,16 +84,28 @@ dotest "-c 4 -w 4 00 964213" "2"
 dotest "-c 4 -w 4 -d 6 00 964213" "2"
 dotest "-c 4 -w 4 -d 7 00 964213" "2" fail
 dotest "-c 4 -d 7 00 964213" "2" fail
-dotest "--totp 3132333435363738393031323334353637383930 -N @59 -d8" "94287082"
-dotest "--totp 3132333435363738393031323334353637383930 -N @1111111109 -d8" "07081804"
-dotest "--totp 3132333435363738393031323334353637383930 -N @1111111111 -d8" "14050471"
-dotest "--totp 3132333435363738393031323334353637383930 -N @1234567890 -d8" "89005924"
-dotest "--totp 3132333435363738393031323334353637383930 -N @2000000000 -d8" "69279037"
+dotest "--totp $sha1key -N @59 -d8" "94287082"
+dotest "--totp=sha256 $sha256key -N @59 -d8" "46119246"
+dotest "--totp=sha512 $sha512key -N @59 -d8" "90693936"
+dotest "--totp $sha1key -N @1111111109 -d8" "07081804"
+dotest "--totp=sha256 $sha256key -N @1111111109 -d8" "68084774"
+dotest "--totp=sha512 $sha512key -N @1111111109 -d8" "25091201"
+dotest "--totp $sha1key -N @1111111111 -d8" "14050471"
+dotest "--totp=sha256 $sha256key -N @1111111111 -d8" "67062674"
+dotest "--totp=sha512 $sha512key -N @1111111111 -d8" "99943326"
+dotest "--totp $sha1key -N @1234567890 -d8" "89005924"
+dotest "--totp=sha256 $sha256key -N @1234567890 -d8" "91819424"
+dotest "--totp=sha512 $sha512key -N @1234567890 -d8" "93441116"
+dotest "--totp $sha1key -N @2000000000 -d8" "69279037"
+dotest "--totp=sha256 $sha256key -N @2000000000 -d8" "90698825"
+dotest "--totp=sha512 $sha512key -N @2000000000 -d8" "38618901"
 dotest "--totp 00 --time-step-size=45m --start-time=now --now=now" "328482"
 dotest "--totp 00 --time-step-size=1m --start-time=now --now=hour -d7" "4511481"
-dotest "--totp --now @1111111109 -w 10 3132333435363738393031323334353637383930 07081804" "0"
-dotest "--totp --now @1111111109 -w 10 3132333435363738393031323334353637383930 14050471" "1"
-dotest "--totp --now @1111111109 -w 5 3132333435363738393031323334353637383930" "081804 050471 266759 306183 466594 754889"
+dotest "--totp --now @1111111109 -w 10 $sha1key 07081804" "0"
+dotest "--totp=sha256 --now @1111111109 -w 10 $sha256key 68084774" "0"
+dotest "--totp --now @1111111109 -w 10 $sha1key 14050471" "1"
+dotest "--totp --now @1111111109 -w 5 $sha1key" "081804 050471 266759 306183 466594 754889"
+dotest "--totp=sha256 --now @1111111109 -w 5 $sha256key" "084774 062674 267535 096086 328915 956967"
 dotest "--hotp --counter 1099511627776 00" "363425"
 
 exit 0
